import React, { Component, Fragment } from 'react'
import PropTypes from 'prop-types'
import OAC from 'oneaccount-js-core'
import 'oneaccount-js-core/dist/oneaccount.css'
import './styles.css'

// TODO: make easier to change styles
// TODO: move styles.css to oneaccount.css in library
// TODO: Add onClick to qrcode
// TODO: Add option to enable helper menu to show instruction of installation and usage
// TODO: Add option to change logo for `login` text or any other text
// TODO: make use of children property of react to inject html

const TitleSVG = ({ color }) => (
  <svg fill={color} viewBox="0 0 320 61" xmlns="http://www.w3.org/2000/svg">
    <text text-anchor="middle" x="50%" y="20">
      Download One Account mobile
    </text>
    <text text-anchor="middle" x="50%" y="40">
      app and scan the QR code
    </text>
    <text text-anchor="middle" fontSize="10px" x="50%" y="60">
      or click the QR code if One Account is installed on current device
    </text>
  </svg>
)

const ANIMATION_DURATION = 3000
class OneAccount extends Component {
  constructor(props) {
    super(props)
    this.floatingModeContainer = React.createRef()
    this.floatingModeButton = React.createRef()
    this.updateQRCode = this.updateQRCode.bind(this)

    this.toVDOM = this.toVDOM.bind(this)
    this.toggle = this.toggle.bind(this)
    const { floatingMode } = this.props
    this.state = {
      isOpen: !floatingMode,
      showSpinner: false,
      running: false
    }
    this.base = 'oneaccount-react'

    if (this.props.updateInterval < 0) this.props.updateInterval = 0

    this.oac = new OAC(this.props.externalId, {
      color: this.props.color,
      level: this.props.level,
      domain: this.props.domain
    })

    this.qrcode = this.oac.QRCode()
    this.props.onUpdate(this.oac.UUID())
  }

  refreshOAC() {
    this.qrcode = this.oac.QRCode()
    this.props.onUpdate(this.oac.UUID())
  }

  hideOnClickOutside(element) {
    const outsideClickListener = event => {
      if (
        !this.floatingModeContainer.current.contains(event.target) &&
        !this.floatingModeButton.current.contains(event.target)
      ) {
        // or use: event.target.closest(selector) === null
        this.stop()
        this.setState({ isOpen: false })
        removeClickListener()
      }
    }

    const removeClickListener = () => {
      document.removeEventListener('click', outsideClickListener)
    }

    document.addEventListener('click', outsideClickListener)
  }

  componentDidMount() {
    // document.addEventListener('click', () => {})

    if (this.props.floatingMode) return
    this.start()
  }

  start() {
    if (this.state.running) return
    this.setState({ running: true })
    this.timerQRCode = setInterval(
      () => this.updateQRCode(),
      this.props.updateInterval + ANIMATION_DURATION
    )
  }

  componentWillUnmount() {
    this.stop()
  }
  // componentWillUpdate(nextProps, nextState) {
  //   console.log(1, nextProps, nextState);
  // }

  updateQRCode() {
    let count = ANIMATION_DURATION / 1000
    this.setState({ showSpinner: true })
    this.timerQRCodeUpdate = setInterval(() => {
      if (count <= 1) {
        this.refreshOAC()
        this.setState({ showSpinner: false })
        clearInterval(this.timerQRCodeUpdate)
      }
      count--
    }, 1000)
  }

  stop() {
    this.refreshOAC()
    this.setState({ showSpinner: false })
    this.setState({ running: false })
    clearInterval(this.timerQRCode)
    clearInterval(this.timerQRCodeUpdate)
  }

  toggle() {
    if (!this.props.floatingMode) return
    if (!this.state.isOpen) {
      this.start()
      this.hideOnClickOutside()
    } else {
      this.stop()
    }
    this.setState({ isOpen: !this.state.isOpen })
  }

  toVDOM({ svg, onClick, className = '' } = {}) {
    return (
      <svg
        className={svg.className + ' ' + className}
        viewBox={svg.viewBox}
        onClick={onClick}
      >
        {svg.paths.map((obj, index) => {
          if (obj.type === 'circle') {
            return (
              <circle
                key={index}
                stroke={obj.stroke}
                strokeWidth={obj.strokeWidth}
                fill={obj.fill}
                cx={obj.cx}
                cy={obj.cy}
                r={obj.r}
              />
            )
          } else {
            return <path key={index} fill={obj.fill} d={obj.path} />
          }
        })}
      </svg>
    )
  }

  render() {
    const { floatingMode, title, icons } = this.props
    const color = this.props.color || '#3C444F'
    let { isOpen, showSpinner } = this.state

    const logo = this.toVDOM({ svg: this.oac.Logo() })
    const roundLogo = this.toVDOM({
      svg: this.oac.Logo(undefined, 'oneaccount-logo-round')
    })
    const spinner = this.toVDOM({
      svg: this.oac.Spinner(),
      className: showSpinner ? 'show-spinner' : ''
    })
    const qrcode = this.toVDOM({ svg: this.qrcode })
    const iconAndroid = this.toVDOM({ svg: this.oac.IconAndroid() })
    const iconIOS = this.toVDOM({ svg: this.oac.IconIOS() })

    let containerClass = `${floatingMode ? `${this.base}-floating-mode ` : ''}`
    if (floatingMode && isOpen) {
      containerClass += `${this.base}-show-qrcode`
    }

    // open mac app
    // TODO: handle this correctly!
    const link = `oneaccount://oneaccount.app?externalId=${
      this.props.externalId
      }&uuid=${this.oac.UUID()}&expires=${this.oac.Expires()}`

    return (
      <div className="oneaccount">
        {floatingMode && (
          <div
            ref={this.floatingModeButton}
            style={{ background: color }}
            onClick={this.toggle}
            className={`${this.base}-logo-wrapper`}
          >
            {floatingMode.logo || roundLogo}
          </div>
        )}
        {/* TODO: move these styles to css*/}
        {/* maybe refactor this to use portal using oneaccount? */}
        {/* https://reactjs.org/docs/portals.html*/}
        {/* https://reactjs.org/blog/2017/11/28/react-v16.2.0-fragment-support.html */}
        <div ref={this.floatingModeContainer} className={containerClass}>
          {this.props.children || (
            <Fragment>
              {title && (
                <p className="oneaccount-title">
                  {title !== true ? title : <TitleSVG color={color} />}
                </p>
              )}
              {icons &&
                (icons !== true ? (
                  icons
                ) : (
                    <Fragment>
                      <a href="#">{iconIOS}</a>
                      <a href="#">{iconAndroid}</a>
                    </Fragment>
                  ))}
            </Fragment>
          )}
          <div
            className="oneaccount-container"
            onClick={() => {
              if (!this.state.showSpinner) window.location = link
            }}
          >
            {this.props.logo || logo}
            {spinner}
            {qrcode}
          </div>
        </div>
      </div>
    )
  }
}

OneAccount.propTypes = {
  externalId: PropTypes.string.isRequired,
  updateInterval: PropTypes.number,
  level: PropTypes.string,
  color: PropTypes.string,
  // NOTE: maybe temp
  domain: PropTypes.string,
  logo: PropTypes.element,
  floatingMode: PropTypes.oneOfType([
    PropTypes.shape({
      logo: PropTypes.element
    }),
    PropTypes.bool
  ]),
  title: PropTypes.oneOfType([
    PropTypes.string,
    PropTypes.element,
    PropTypes.bool
  ]),
  icons: PropTypes.oneOfType([PropTypes.bool, PropTypes.element]),
  onUpdate: PropTypes.func
}

OneAccount.defaultProps = {
  ...Component.defaultProps,
  logo: null,
  floatingMode: false,
  updateInterval: 60 * 1000,
  level: 'M',
  color: '#3C444F',
  onUpdate: () => { }
}

export default OneAccount